@@ -360,6 +360,14 @@ static inline int splunk_format(const void *in_buf, size_t in_bytes,
         if (ctx->event_key) {
             /* Pack the value of a event key */
             ret = pack_event_key(ctx, &mp_pck, &tm, map, tag, tag_len);
+            if (ret != 0) {
+                /*
+                 * if pack_event_key fails due to missing content in the
+                 * record, we just warn the user and try to pack it
+                 * as a normal map.
+                 */
+                ret = pack_map(ctx, &mp_pck, &tm, map, tag, tag_len);
+            }
         }
         else {
             /* Pack as a map */
@@ -372,7 +380,7 @@ static inline int splunk_format(const void *in_buf, size_t in_bytes,
             err = flb_msgpack_to_json_str(2048, &map);
             if (err) {
                 /* Print error and continue processing other records */
-                flb_plg_warn(ctx->ins, "could not process record: %s", err);
+                flb_plg_warn(ctx->ins, "could not process or pack record: %s", err);
                 msgpack_sbuffer_destroy(&mp_sbuf);
                 flb_free(err);
             }
